#!/usr/bin/perl
#
#  Copyright (c) 2004, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#   * Neither the name of SWITCH nor the names of its contributors may be
#     used to endorse or promote products derived from this software without
#     specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.
#
#  $Author:$
#
#  $Id:$
#
#  $LastChangedRevision:$

package NfProfile;

use strict;
use Sys::Syslog; 
Sys::Syslog::setlogsock('unix');

use NfSen;
use NfSenRRD;
use Log;

#
# Default profile description
my @ProfileTag = ( 
	"# \n",
);


local $SIG{'__DIE__'} = sub {
	my $message = shift;

	syslog("err","PANIC!: I'm dying: '$message'");

}; # End of __DIE__ 


sub ProfileHistory {
	my $profileref = shift;

	my $name   = $$profileref{'name'};
	my $tstart = $$profileref{'tstart'};
	my $tend   = $$profileref{'tend'};
	my $filter = "$NfConf::BASEDIR/profiles/$name/$$profileref{'filter'}";

	my @sourcelist = split /:/, $$profileref{'sourcelist'};

	# we have to process that many files:
	my $numfiles = ((( $tend - $tstart ) / 300 ) + 1 ) * scalar @sourcelist;
	
	$$profileref{'status'} = 'built 0';
	if ( !NfSen::WriteProfile($profileref) ) {
		syslog('err', "Error writing profile '$name': $Log::ERROR");
		return;
	}

	my $counter  = 0;
	my $progress = 0;
	my $percent  = $numfiles / 100;

	syslog('info', "ProfileHistory for profile '$name': $numfiles files."),
	# Update the profile status every .1% of all files, at least every 2 files
	# more often does not makes senes
	my $modulo	 = int ($percent / 10) < 2 ? 2 : int ($percent / 10);
	my $profile_size = 0;
	for ( my $t = $tstart; $t <= $tend; $t += 300 ) {
		my $iso = NfSen::UNIX2ISO($t);
		my %statinfo    = {};
		foreach my $source ( @sourcelist ) {
			$counter++;
			my $infile  = "$NfConf::BASEDIR/profiles/live/$source/nfcapd.$iso";
			my $outfile = "$NfConf::BASEDIR/profiles/$name/$source/nfcapd.$iso";

			my $completed = sprintf "%.1f%%", $counter / $percent;

			if ( ($counter % $modulo ) == 0 ) {
				$$profileref{'status'} = "built $completed\n";
				NfSen::WriteProfile($profileref);
				syslog('info', "Build Profile '$name':     Completed: $completed\%"),
			}

			if ( system( "$NfConf::PREFIX/nfdump -r $infile -w $outfile -f $filter" ) != 0 ) {
            	my $exit_value  = $? >> 8;
            	my $signal_num  = $? & 127;
            	my $dumped_core = $? & 128;
            	syslog('err', "nfdump unexpectedly died: Exit: $exit_value, Signal: $signal_num, Core: $dumped_core\n");
				next; #	source in foreach $source
			} 
			# array element 7 contains the size in bytes
			$profile_size += (stat($outfile))[7];

			# get statinfo from this slot
			$statinfo{$source} = NfSen::ReadStatInfo($profileref, $source, $iso);
			if ( defined $Log::ERROR ) {
				syslog('err', $Log::ERROR);
				next; #	source in foreach $source
			}
		}

		# update each RRD database
		foreach my $dbtype ( @NfSenRRD::RRDdb ) {
			my @_tmp = ();
			foreach my $source ( @sourcelist ) {
				# if the stat file was missing for this source ( should
				# not happen anyway, but who knows ... ) use '0'
				push @_tmp, $statinfo{$source}{$dbtype} == - 1 ? 0 : $statinfo{$source}{$dbtype};
			}
			NfSenRRD::UpdateDB("$NfConf::BASEDIR/profiles/$name", $dbtype, $t,
				$$profileref{'sourcelist'}, join ':', @_tmp);
			if ( $Log::ERROR ) {
				syslog('err', "ERROR Update RRD time: '$iso', db: '$dbtype', profile: '$name': $Log::ERROR");
			}
		}
	}
	syslog('info', "ProfileHistory: Done."),

	return $profile_size;

} # end of ProfileHistory

#
# ListProfiles is called only from cmd line
sub ListProfile {
	my $socket 	= shift;
	my $opts 	= shift;

	my $profile  = exists $$opts{'profile'} ? $$opts{'profile'} : undef;
	my $raw_mode = exists $$opts{'quiet'};

	if ( defined $profile ) {
		my $err = undef;
		if ( $profile =~ /[^A-Za-z0-9\-+_]+/ ) {
			$err = "ERR Illegal characters in profile name '$profile'!\n";
		} elsif ( !-d "$NfConf::BASEDIR/profiles/$profile") {
			$err = "ERR Profile '$profile' does not exists\n";
		} elsif ( !-f "$NfConf::BASEDIR/profiles/$profile/profile.dat") {
			$err = "ERR Missing profile descriptor file of profile '$profile'\n";
		}
		if ( defined $err ) {
			print $socket $err;
			return;
		}
	} 
	
	my @AllProfiles;
	if ( defined $profile ) {
		push @AllProfiles, $profile;
	} else {
		@AllProfiles = NfSen::ProfileList();
		if ( defined $Log::ERROR ) {
			print $socket "ERR $Log::ERROR\n";
			return;
		}
	}

	foreach my $profilename ( @AllProfiles ) {
		if ( !NfSen::InfoFileExists($profilename) ) {
			print $socket "No info for profile '$profilename'\n";
			next;
		}
		my %profileinfo = NfSen::ReadProfile($profilename);
		if ( $profileinfo{'status'} eq 'empty' ) {
			print $socket "ERR Profile '$profilename': $Log::ERROR\n";
			next;
		}

		if ( $raw_mode ) {
			# raw output format
			foreach my $key ( keys %profileinfo ) {
				if ( $key eq '_comment_' ) {
					my $tmp = join '', @{$profileinfo{$key}};
					# prepare sending the multiline profile description in a single line
					# escape ', | and \n
					$tmp =~ s/\\/\\134/g;	# escape escape char
					$tmp =~ s/'/\\047/g;	# escape '
					$tmp =~ s/\|/\\174/g;	# escape |
					$tmp =~ s/\n/\\012/g;	# escape \n
					print $socket "description|$tmp\n";
				} else {
					print $socket  "$key|$profileinfo{$key}\n";
				}
			}
		} else {
			# formated output
			my $type = $profileinfo{'type'};

			print $socket  @{$profileinfo{'_comment_'}} if  exists $profileinfo{'_comment_'};
			print $socket  "name\t$profilename\n";
			print $socket  "tstart\t"  . scalar localtime($profileinfo{'tstart'}) . "\n";
			print $socket  "tend\t"    . scalar localtime($profileinfo{'tend'}) . "\n";
			print $socket  "updated\t" . scalar localtime($profileinfo{'updated'}) . "\n";
			if ( $type == 0 ) {
				print $socket  "filter\t<none>\n";
			} else {
				print $socket  "filter\t$profileinfo{'filter'}\n";
			}
			my ($d, $h);
			$d = int($profileinfo{'expire'} / 24 );
			$h = $profileinfo{'expire'} % 24;
			my ( $d_str, $h_str );
			$d_str = $h_str = '';
			if ( $d ) {
				$d_str = "$d day";
				if ( $d == 0 || $d > 1 ) {
					$d_str .= "s";
				}
				$d_str .= " ";
			}
			$h_str = "$h hour";
			if ( $h == 0 || $h > 1 ) {
				$h_str .= "s";
			}
			print $socket  "expire\t${d_str}${h_str}\n";
			print $socket  "size\t" .   NfSen::ScaledValue($profileinfo{'size'}) . "\n";
			print $socket  "maxsize\t". NfSen::ScaledValue($profileinfo{'maxsize'}) . "\n";
			print $socket  "sources\t$profileinfo{'sourcelist'}\n";
			print $socket  "type\tlive\n"	     if $type == 0;
			print $socket  "type\thistory\n"	 if $type == 1;
			print $socket  "type\tcontinuous\n"  if $type == 2;
			print $socket  "locked\t$profileinfo{'locked'}\n";
			print $socket  "status\t$profileinfo{'status'}\n";
			print $socket  "\n";
		}
	}
	
	print $socket "OK Command completed\n" if exists $$opts{'quiet'};

} # End of ListProfiles

sub AddProfile {
	my $socket = shift;
	my $opts   = shift;

	# both options can be set at the same time
	my $history_profile    = 0;	# Profile start back in time; needs to be builded
	my $continuous_profile = 0;	# Profile continuous in time
	
	my $profile;
	# Parameter checking
	if ( exists $$opts{'profile'} ) {
		$profile	= $$opts{'profile'};
		if ( $profile =~ /[^A-Za-z0-9\-+_]+/ ) {
			print $socket "ERR Illegal characters in profile name '$profile'!\n";
			return;
		}
		if ( NfSen::InfoFileExists($profile) ) {
			print $socket "ERR Profile '$profile' already exists!\n";
			return;
		}
	} else {
		print $socket "ERR Missing profile name!\n";
		return;
	}

	my $now 	= time();
	my $tlast	= $now - ( $now % 300 ) - 300;
	my $tstart	= $tlast;
	my $tend 	= $tstart;

	if ( exists $$opts{'tstart'} ) {
		$tstart = $$opts{'tstart'};
		if ( !NfSen::ValidISO($tstart) ) {
			print $socket "ERR Time format wrong '$tstart'!\n";
			return;
		}
		$tstart = NfSen::ISO2UNIX($tstart);
		$history_profile = 1;
	} else {
		# otherwise we have a ontinuous profile
		$continuous_profile = 1;
	}

	if ( exists $$opts{'tend'} ) {
		$tend = $$opts{'tend'};
		if ( !NfSen::ValidISO($tend) ) {
			print $socket "ERR Time format wrong '$tend'!\n";
			return;
		}
		$tend = NfSen::ISO2UNIX($tend);
		$history_profile = 1;
	} else {
		# otherwise we have a ontinuous profile
		$continuous_profile = 1;
	}

	# new profile must fit into live profile
	my %liveprofile = NfSen::ReadProfile('live');
	if ( $liveprofile{'status'} eq 'empty' ) {
		# Could not read live profile
		print $socket "ERR profile 'live': $Log::ERROR\n";	
		return;
	}
	if ( $history_profile && ($tstart < $liveprofile{'tstart'}) ) {
		print $socket "ERR '$$opts{'tstart'}' not within 'live' profile time window\n";
		return;
	}
	if ( $history_profile && ($tend > $liveprofile{'tend'}) ) {
		print $socket "ERR '$$opts{'tend'}' not within 'live' profile time window\n";
		return;
	}

	# prevent overflow to future timestamps
	$tend = $tlast if $tend > $tlast;	

	# expire time
	my $lifetime   = exists $$opts{'expire'}  ? NfSen::ParseExpire($$opts{'expire'}) : 0;
	if ( $lifetime < 0 ) {
		print $socket "ERR Unknown expire time '$$opts{'expire'}'\n";
		return;
	}

	# max size
	my $maxsize	= exists $$opts{'maxsize'} ? NfSen::ParseMaxsize($$opts{'maxsize'}) : 0;
	if ( $maxsize < 0 ) {
		print $socket "ERR Unknown max size '$$opts{'maxsize'}'\n";
		return;
	}

	# source list
	my $sourcelist = exists $$opts{'sources'} ? $$opts{'sources'} : join ':', keys %NfConf::sources;
	foreach my $source ( split /:/, $sourcelist ) {
		if ( !exists $NfConf::sources{$source} ) {
			print $socket "ERR Unknown source '$source'\n";
			return;
		}
	}
	
	# Profile filter:
	# A given 'filter' overwrites the filter in the file 'filterfile'
	my $filter = undef;
	if ( exists $$opts{'filter'} ) {
		$filter = $$opts{'filter'};
	} elsif ( exists $$opts{'filterfile'} ) {
		my $file = $$opts{'filterfile'};
		open(FILTER, "$file" ) or
			syslog('err', "Can't open filter file '$filter': $!"),
			print $socket "ERR Can't open filter file '$filter': $!\n",
			return;
		$filter = join '', <FILTER>;
		close FILTER;
	}
	if ( !defined $filter || $filter eq '' ) {
		print $socket "ERR Missing profile filter\n";
		return;
	}
	my @out = NfSen::VerifyFilter($filter);
	if ( scalar @out > 0 ) {
		print $socket "Filter syntax error:\n";
		foreach my $line ( @out ) {
			print $socket $line;
		}
		print $socket "ERR Filter contains errors\n";
		return;
	}
	
	# Do the work now:
	umask 0002;
	mkdir "$NfConf::BASEDIR/profiles/$profile" or
		syslog("err", "Can't create profile directory '$NfConf::BASEDIR/profiles/$profile': $!"),
		print $socket "ERR Can't create profile directory: $!\n",
		return;

	foreach my $source ( split /:/, $sourcelist  ) {
		mkdir "$NfConf::BASEDIR/profiles/$profile/$source" or
			print $socket "ERR Can't create source directory: $!\n",
			return;
	}

	my %profileinfo;
	if ( exists $$opts{'description'}  ) {
		my @_tmp = split /\n/, $$opts{'description'};
		@_tmp = map "$_\n", @_tmp;
		$profileinfo{'_comment_'} 	= \@_tmp;
	} else {
		$profileinfo{'_comment_'} 	= \@ProfileTag;
	}
	$profileinfo{'name'}		= $profile;
	$profileinfo{'tstart'} 		= $tstart;
	$profileinfo{'tend'} 		= $tend;
	$profileinfo{'sourcelist'}	= $sourcelist;

	# the first slot to be updated is 'updated' + 300
	$profileinfo{'updated'}		= $tstart - 300;	

	# expiring profiles makes only sense for continous profiles
	$profileinfo{'expire'} 		= $continuous_profile ? $lifetime : 0;
	$profileinfo{'maxsize'} 	= $continuous_profile ? $maxsize : 0;

	# the profile starts we 0 size
	$profileinfo{'size'} 		= 0;

	# 'filter.txt' triggers nfprofile, otherwise use any other name
	$profileinfo{'filter'}		= $continuous_profile ? "filter.txt" : "history_filter.txt";

	# continuous profile overwrites history profile
	$profileinfo{'type'} 		= 1 if $history_profile;
	$profileinfo{'type'} 		= 2 if $continuous_profile;
	
	# if we need to create the profile from
	# history data, lock the profile until down.
	$profileinfo{'locked'} 		= $history_profile;

	# status of profile
	$profileinfo{'status'}		= 'new';

	# copy the filter to it's file in the profile directory
	open FILTER, ">$NfConf::BASEDIR/profiles/$profile/$profileinfo{'filter'}";
	print FILTER $filter;
	close FILTER;

	if ( !NfSen::WriteProfile(\%profileinfo) ) {
		syslog('err', "Error writing profile '$profile': $Log::ERROR");
		print $socket "ERR writing profile '$profile': $Log::ERROR\n";
		return;
	}
	
	# $tstart is the first value we need in the RRD DB, therefore specify 
	# $tstart - 300 ( 1 slot )
	NfSenRRD::SetupRRD("$NfConf::BASEDIR/profiles/$profile", $tstart - 300, 1);
	if ( defined $Log::ERROR ) {
		syslog('err', "Error creating RRD DB for '$profile': $Log::ERROR");
	}

	if ( $history_profile ) {
    	chdir '/';
		my $pid = fork;
		if ( !defined $pid ) {
			$Log::ERROR = $!;
			syslog('err', "Can't fork: $Log::ERROR");
			print $socket "ERR Can't fork: $Log::ERROR\n";
			return;
		}
		if ( $pid ) { 
			# we are the parent processs
			if ( exists $$opts{'quiet'} ) {
				print $socket "OK Profiling netflow data\n";
			} else {
				ListProfile($socket, $opts);
			}
			open PID, ">$NfConf::BASEDIR/var/run/build.${pid}" || 
				warn "Can't open pid file: $!";
            print PID "$pid\n";
            close PID;
			return;
		}
		# the child starts to build the profile from history data
		close STDIN;
		close STDOUT;
		close STDERR;
		$profileinfo{'size'} 	= ProfileHistory(\%profileinfo);
		$profileinfo{'updated'}	= $profileinfo{'tend'};
		unlink "$NfConf::BASEDIR/var/run/build.$$";
	}

	syslog('debug', "Graph update profile: $profile, Time: $profileinfo{'tend'}.");
	if ( NfSenRRD::UpdateGraphs($profile, $profileinfo{'tend'}, 1) ) {
		syslog('err', "Error graph update: $Log::ERROR");
	}

	$profileinfo{'status'}	= 'OK';	

	# unlock profile
	$profileinfo{'locked'} 		= 0;

	NfSen::WriteProfile(\%profileinfo);
	
	if ( exists $$opts{'quiet'} ) {
		print $socket "OK Add profile completed\n" if exists $$opts{'quiet'};
	} else {
		ListProfile($socket, $opts);
	}

} # End of AddProfile

sub DeleteProfile {
	my $socket	= shift;
	my $opts 	= shift;

	my $profile;
	# Parameter checking
	if ( exists $$opts{'profile'} ) {
		$profile	= $$opts{'profile'};
		if ( $profile =~ /[^A-Za-z0-9\-+_]+/ ) {
			print $socket "ERR Illegal characters in profile name '$profile'!\n";
			return;
		}
		if ( !NfSen::InfoFileExists($profile) ) {
			print $socket "ERR Profile '$profile' does not exists!\n";
			return;
		}
	} else {
		print $socket "ERR Missing profile name!\n";
		return;
	}

	if ( $profile eq 'live' ) {
		print $socket "ERR Don't want to delete profile 'live'!\n";
		return;
	}

	my %profileinfo = NfSen::ReadProfile($profile);
	if ( $profileinfo{'status'} eq 'empty' ) {
		# Could not read live profile
		print $socket "ERR Profile '$profile': $Log::ERROR\n";	
		return;
	}
	if ( $profileinfo{'locked'} ) {
		print $socket "ERR Profile '$profile' is locked and can not be deleted now!\n";
		return;
	}
	if ( $profileinfo{'status'} ne 'OK' ) {
		print $socket "ERR Profile '$profile' is not in status 'OK' and can not be deleted now!\n";
		return;
	}

	if ( !rename "$NfConf::BASEDIR/profiles/$profile", "$NfConf::BASEDIR/profiles/.$profile" ) {
		print $socket "ERR Failed to rename profile '$profile' in order to delete: $!\n";
		return;
	}
	system("/bin/rm -rf $NfConf::BASEDIR/profiles/.$profile &");

	print $socket "OK Profile '$profile' deleted.\n" if exists $$opts{'quiet'};

} # End of DeleteProfile

sub ModifyProfile {
	my $socket	= shift;
	my $opts 	= shift;

	my $profile;
	# Parameter checking
	if ( exists $$opts{'profile'} ) {
		$profile	= $$opts{'profile'};
		if ( $profile =~ /[^A-Za-z0-9\-+_]+/ ) {
			print $socket "ERR Illegal characters in profile name '$profile'!\n";
			return;
		}
		if ( !NfSen::InfoFileExists($profile) ) {
			print $socket "ERR Profile '$profile' does not exists!\n";
			return;
		}
	} else {
		print $socket "ERR Missing profile name!\n";
		return;
	}
	my %profileinfo = NfSen::ReadProfile($profile);
	if ( $profileinfo{'status'} eq 'empty' ) {
		# Could not read profile
		print $socket "ERR Profile '$profile': $Log::ERROR\n";	
		return;
	}

	my $continuous_profile = $profileinfo{'type'} == 2 || $profileinfo{'type'} == 0;

	my $changed = 0;

	if ( exists $$opts{'lock'} ) {
		$profileinfo{'locked'} = 0 if $$opts{'lock'} == 0;
		$profileinfo{'locked'} = 1 if $$opts{'lock'} == 1;
		$changed = 1;
	}

	if ( $continuous_profile ) {
		# these changes make only sense for continuous profiles
		# expire time
		if ( exists $$opts{'expire'} ) {
			my $lifetime   = NfSen::ParseExpire($$opts{'expire'});
			if ( $lifetime < 0 ) {
				print $socket "ERR Unknown expire time '$$opts{'expire'}'\n";
				return;
			}
			$profileinfo{'expire'} = $lifetime;
			$changed = 1;
		}
	
		# max size
		if ( exists $$opts{'maxsize'} ) {
 			my $maxsize = NfSen::ParseMaxsize($$opts{'maxsize'});
			if ( $maxsize < 0 ) {
				print $socket "ERR Unknown max size '$$opts{'maxsize'}'\n";
				return;
			}
			$profileinfo{'maxsize'}	= $maxsize;
			$changed = 1;
		}
	}
	
	if ( $changed ) {
		if ( !NfSen::WriteProfile(\%profileinfo) ) {
			syslog('err', "Error writing profile '$profile': $Log::ERROR");
			print $socket "ERR writing profile '$profile': $Log::ERROR\n";
			return;
		}
		if ( exists $$opts{'quiet'} ) {
			print $socket "OK profile modified\n" if exists $$opts{'quiet'};
			return;
		} else {
			ListProfile($socket, $opts);
			return;
		}
	} else {
		print $socket "OK Nothing modified\n" if exists $$opts{'quiet'};
	}

} # End of ModifyProfile

sub RebuildProfile {
	my $socket	= shift;
	my $opts 	= shift;

	my $profile;
	# Parameter checking
	if ( exists $$opts{'profile'} ) {
		$profile	= $$opts{'profile'};
		if ( $profile =~ /[^A-Za-z0-9\-+_]+/ ) {
			print $socket "ERR Illegal characters in profile name '$profile'!";
			return;
		}
		if ( !NfSen::InfoFileExists($profile) ) {
			print $socket "ERR Profile '$profile' does not exists!\n";
			return;
		}
	} else {
		print $socket "ERR Missing profile name!\n";
		return;
	}

	print $socket "Query your data files ... please wait\n" unless exists $$opts{'quiet'};

	opendir(PROFILEDIR, "$NfConf::BASEDIR/profiles/$profile" ) or
		syslog('err', "Rebuild: Can't open profile directory '$profile': $!"),
		print $socket "ERR Can't opendir\n",
		return;

	my @AllSources = grep { -d "$NfConf::BASEDIR/profiles/$profile/$_" } readdir(PROFILEDIR);
	closedir PROFILEDIR;

	my @sourcelist;
	foreach my $source ( @AllSources ) {
		push @sourcelist, $source if exists $NfConf::sources{$source};
	}

	if ( scalar @sourcelist == 0 ) {
		syslog('err', "No valid sources found in profile '$profile'");
		print $socket "ERR No valid sources found in profile '$profile'\n";
		return;
	}

	opendir(PROFILESRC, "$NfConf::BASEDIR/profiles/$profile/$sourcelist[0]" ) or
		syslog('err', "Rebuild: Can't open profile directory '$profile/$sourcelist[0]': $!"),
		print $socket "ERR Can't opendir '$profile/$sourcelist[0]'\n",
		return;

	my @files = grep { /^nfcapd\.\d+$/ && 
				-f "$NfConf::BASEDIR/profiles/$profile/$sourcelist[0]/$_" } readdir(PROFILESRC);

	closedir PROFILESRC;
	if ( scalar @files == 0 ) {
		syslog("info",  "Rebuild: No data file found in profile '$profile'");
		print $socket "OK No data file found in profile '$profile'\n" if exists $$opts{'quiet'};
		return;
	}

	my @sortedfiles = sort @files;
	my ( $first ) = (shift @sortedfiles) =~ /nfcapd\.(\d+)/;
	my ( $last )  = (pop @sortedfiles)   =~ /nfcapd\.(\d+)/;

	print $socket "done.\n" unless exists $$opts{'quiet'};

	my %profileinfo = NfSen::LockProfile($profile);
	if ( $profileinfo{'status'} eq 'empty' ) {
		# Could not read live profile
		print $socket "ERR Profile '$profile': $Log::ERROR\n";	
		return;
	}
	my $continuous_profile = -f "$NfConf::BASEDIR/profiles/$profile/filter.txt";
	my $history_profile	= -f "$NfConf::BASEDIR/profiles/$profile/history_filter.txt";

	if ( $profileinfo{'name'} eq '' ) {
		$profileinfo{'_comment_'} 	= \@ProfileTag;
		$profileinfo{'name'}		= $profile;

	} else {
		if ( $profileinfo{'name'} ne $profile ) {
			syslog("warning", "Profile name missmatch - Set '$profileinfo{'name'}' to '$profile'");
			$profileinfo{'name'} = $profile;
		}

	}
	$profileinfo{'tstart'} 		= NfSen::ISO2UNIX($first);
	$profileinfo{'tend'} 		= NfSen::ISO2UNIX($last);
	$profileinfo{'sourcelist'}	= join ':', @sourcelist;
	# 'updated' + 300 -> next update = $tstart
	$profileinfo{'updated'}		= $profileinfo{'tstart'} - 300;	

	# the profile starts with size 0
	$profileinfo{'size'} 		= 0;
	$profileinfo{'maxsize'} 	= defined $profileinfo{'maxsize'} ? $profileinfo{'maxsize'} : 0;
	$profileinfo{'expire'} 		= defined $profileinfo{'expire'} ? $profileinfo{'expire'} : 0;
	$profileinfo{'filter'}		= '';
	$profileinfo{'filter'}		= "filter.txt" if $continuous_profile;
	$profileinfo{'filter'}		= "history_filter.txt" if $history_profile;

	# continuous profile overwrites history profile
	$profileinfo{'type'} 		= 0;
	$profileinfo{'type'} 		= 1 if $history_profile;
	$profileinfo{'type'} 		= 2 if $continuous_profile;
	
	# if we need to create the profile from
	# history data, lock the profile until down.
	$profileinfo{'locked'} 		= 0;

	# state rebuild prevents to run the plugins for each updated timeslot
	$profileinfo{'status'} 		= 'rebuild';

	umask 0002;
	# $tstart is the first value we need in the RRD DB, therefore specify 
	# $tstart - 300 ( 1 slot )
	NfSenRRD::SetupRRD("$NfConf::BASEDIR/profiles/$profile", $profileinfo{'tstart'} - 300, 1);	
	if ( defined $Log::ERROR ) {
		syslog('err', "Error creating RRD DB for '$profile': $Log::ERROR");
	}
	if ( !NfSen::WriteProfile(\%profileinfo) ) {
		syslog('err', "Error writing profile '$profile': $Log::ERROR");
		print $socket "ERR writing profile '$profile': $Log::ERROR\n";
		return;
	}

	if ( exists $$opts{'quiet'} ) {
		print $socket "OK profile modified\n";
		return;
	} else {
		ListProfile($socket,$opts);
		return;
	}

} # End of RebuildProfile

sub QueryProfiles {
	my $socket	= shift;
	my $opts 	= shift;

	foreach my $profile ( NfSen::ProfileList()) {
		print $socket "$profile\n";
	}

	if ( defined $Log::ERROR ) {
		print $socket "ERR $Log::ERROR\n";
	} else {
		print $socket "OK Listing complete\n" if exists $$opts{'quiet'};
	}

	return;

} # End of QueryProfiles
1;
