<?php

//
// Common functions
//

function nfsen_run ( $args ) {

	global $BASEDIR;

	$run = "$BASEDIR/bin/nfsen -q $args";
	$fh = popen("$run 2>&1", 'r');
	$result = array();
	$status = '';
	while ( $line = rtrim(fgets($fh, 1024)) ) {
		if ( preg_match("/^OK|ERR/", $line ) )
			$status = $line;
		else 
			$result[] = $line;
	}
	pclose($fh);

	$_SESSION['nfsen_status'] = $status;
	$_SESSION['nfsen_return'] = $result;
	return preg_match("/^OK/", $status) ? true : false;

} // End of nfsen_run

function UNIX2ISO ($time) {

	$tmp = localtime($time);
	$tstring  = $tmp[5] + 1900;
	$tmp[4]  += 1;
	$tstring .= $tmp[4] < 10 ? "0" . $tmp[4] : $tmp[4];
	$tstring .= $tmp[3] < 10 ? "0" . $tmp[3] : $tmp[3];
	$tstring .= $tmp[2] < 10 ? "0" . $tmp[2] : $tmp[2];
	$tstring .= $tmp[1] < 10 ? "0" . $tmp[1] : $tmp[1];

	return $tstring;

} // End of UNIX2ISO

function ISO2UNIX ($time) {

	// 2004 02 13 12 45 /
	preg_match("/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/", $time, $matches);
	$unixtime = mktime ( $matches[4], $matches[5], 0, $matches[2], $matches[3], $matches[1]);

	return $unixtime;

} // End of ISO2UNIX

function DISPLAY2UNIX($str) {

	$_tmp = preg_replace("/-/", "", $str);

	$_len = strlen($_tmp);
	if ( $_len < 12 || $_len > 12 )
		return -1;

	if ( preg_match("/[^\d]+/", $_tmp ))
		return -1;

	if ( is_numeric($_tmp) && $_tmp > 197001010000 && $_tmp < 203801191414 ) {
		$_tmp = ISO2UNIX($_tmp);
	} else
		$_tmp = -1;

	return $_tmp;

} // End of DISPLAY2UNIX

function UNIX2DISPLAY($time) {
	// converts any UNIX time in display readable format:
	// 2004-02-13-12-45
	
	$iso = UNIX2ISO($time);

	return preg_replace("/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})/", "$1-$2-$3-$4-$5", $iso);

} // End of UNIX2DISPLAY

function ScaleBytes ( $bytes ) {

	$_1KB = 1024.0;
	$_1MB = 1024.0 * $_1KB;
	$_1GB = 1024.0 * $_1MB;
	$_1TB = 1024.0 * $_1GB;

	$scaled = 0;
	if ( $bytes >= $_1TB ) {
		$scaled = $bytes / $_1TB;
		$unit = "TB";
	} elseif ( $bytes >= $_1GB ) {
		$scaled = $bytes / $_1GB;
		$unit = "GB";
	} elseif ( $bytes >= $_1MB ) {
		$scaled = $bytes / $_1MB;
		$unit = "MB";
	} elseif ( $bytes >= $_1KB ) {
		$scaled = $bytes / $_1KB;
		$unit = "KB";
	} else {
		$scaled = $bytes;
		$unit = " B";
	}
	if ( $scaled > 0 )
		$str = sprintf("%5.1f %s", $scaled, $unit);
	else
		$str = "$bytes $unit";

	return preg_replace("/^\s+/", "", $str);
	
} // End of ScaleBytes

//
// Profile related functions
//

function GetProfiles () {

	global $BASEDIR;


	if ( array_key_exists('ProfileList', $_SESSION ) ) {
		if ( array_key_exists('ProfileListUpdate', $_SESSION ) && 
		( time() - $_SESSION['ProfileListUpdate'] < 600 ) ) {
			return $_SESSION['ProfileList'];
		}
	}

	$profiles = nfsen_run("-A");
	if ( $profiles ) {
		$profiles   = $_SESSION['nfsen_return'];
		$profiles[] = "New Profile ...";
	} else {
		$profiles = array();
		$profiles[] = "&lt;No profiles available&gt;";
	}
	$_SESSION['ProfileList'] = $profiles;
	$_SESSION['ProfileListUpdate'] = time();

	return $profiles;

} // End of GetProfiles

function ReadProfile ($profile) {
	global $BASEDIR;

/*
	* this code below can be used to read the profile using nfsen_run
	* As open the profile.dat file and read the profile info directly seems more 
	* efficient, this code remains commented and is not used today
	*
	$args = "-l $profile";
	if ( nfsen_run($args);) 
		return false;

	foreach ( $SESSION['nfsen_return'] as $line ) {
		// Skip rubish lines
		if ( !preg_match("/|/", $line) ) {
			continue;
		}

		list ($key, $value) = explode("|", $line);
		if ( $key == 'description' ) {
			// replace escape chars in description and remove '#' tag on each line
			$value = preg_replace("/\\\\012/", "\n", $value);
			$value = preg_replace("/\\\\047/",  "'", $value);
			$value = preg_replace("/\\\\174/",  "|", $value);
			$value = preg_replace("/\\\\134/", "\\", $value);
			$value = preg_replace("/^# /", "", $value);
			$value = preg_replace("/\n# /", "\n", $value);
			$profileinfo['comment'] = explode("\n", $value);
		} else 
			$profileinfo[$key] = $value;
	}
	return $profileinfo;
*/

	$fh = @fopen("$BASEDIR/profiles/$profile/profile.dat", "r");
	if ( !$fh ) {
		return array();
	}

	$comment = array();
	while ( $line = rtrim(fgets($fh, 1024)) ) {

		// grab comment
		if ( preg_match("/^#/", $line) ) {
			$comment[] = preg_replace("/^#\s{0,1}/", "", $line);
			continue;
		}

		// Skip empty lines
		if ( !preg_match("/=/", $line) ) {
			continue;
		}

		// Read profile info
		$line = preg_replace("/\s*=\s*/", '=', $line);
		list ($key, $value) = explode("=", $line);
		$profileinfo[$key] = $value;
	}
	fclose($fh);
	$profileinfo['comment'] = $comment;

	return $profileinfo;

} // End of ReadProfile

function ShowMessages () {

	if ( array_key_exists('warning', $_SESSION ) ) {
		print "<h3 class='warnstring'>WARNING: " . $_SESSION['warning'] . "!</h3>\n";
		unset($_SESSION['warning']);
	}

	if ( array_key_exists('error', $_SESSION ) ) {
		print "<h3 class='errstring'>ERROR: " . $_SESSION['error'] . "!</h3>\n";
		unset($_SESSION['error']);
	}

} // End of ShowMessages

function ReadStat($source) {

	global $BASEDIR;

	// indices into array
	// 0  Time: 1075888800
	// 1  Ident: swiix1
	// 2  Flows: 2010696
	// 3  Flows_tcp: 2010696
	// 4  Flows_udp: 0
	// 5  Flows_icmp: 0
	// 6  Flows_other: 0
	// 7  Packets: 32534108
	// 8  Packets_tcp: 32534108
	// 9  Packets_udp: 0
	// 10 Packets_icmp: 0
	// 11 Packets_other: 0
	// 12 Bytes: 19871469109
	// 13 Bytes_tcp: 19871469109
	// 14 Bytes_udp: 0
	// 15 Bytes_icmp: 0
	// 16 Bytes_other: 0
	// 17 First: 1075887894
	// 18 Last: 1075889096

	for ( $i=0; $i<=18; $i++ )
		$SumStat[$i] = 0;
	for ( $t=$_SESSION['tleft']; $t <= $_SESSION['tright']; $t += 300 ) {
		$tslot = UNIX2ISO($t);
		$profile = $_SESSION['profile'];
		$statfile = "$BASEDIR/profiles/$profile/$source/nfcapd.$tslot.stat";
		$fh = @fopen("$statfile", "r");
		if ( !$fh ) {
			return array();
		}
	
		$statinfo = array();
		while ( $line = rtrim(fgets($fh, 1024)) ) {
			list ($dummy, $value) = explode(": ", $line);
			$statinfo[] = $value;
		}
		fclose($fh);
		for ( $i=2; $i<17; $i++) {
			$SumStat[$i] += $statinfo[$i];
		}
	}

	return $SumStat;

} // End of ReadStat 

function DefaultFilters () {

	global $BASEDIR;

	$FilterDir = "$BASEDIR/var/filters";
	$dh = opendir($FilterDir);

	if ( !$dh ) {
		// No filter dir. return silently => no default filters
		return;
	}
	
	$filters = array();
	while (($entry = readdir($dh)) !== false) {
		if ( is_file("$FilterDir/$entry") ) {
			$filters[] = $entry;
		}
	}
	closedir($dh);
	$_SESSION['defaultfilters'] = $filters;

} // End of DefaultFilters

/*
 * Parse the expire input string and set new_expire as the number of hours
 * for this profiles new expire value.
 * valid form:
 *	<num>				number of hours
 * 	<num> h|hour|hours	number of hours
 *	<num> d|day|days	number of days
 *	<num> d|day|days <num> h|hour|hours	combination days and hours
 * returns the number of hours of new expire or -1 if string unparsable
 */
function ParseExpire ($str) {

	$new_expire = 0;
	$valid		= 0;	// we found a valid string

	// a single number: add 'h'
	if ( is_numeric($str) ) {
		$str = (int)$str;
		$str .= "h";
	}

	// normalize the input string. Replace 'never' with 0h
	$str  = preg_replace("/never/i", "0h", $str);

	// normalize the input string. Replace all day, days with 'd'
	$str  = preg_replace("/days/i", "d", $str);
	$str  = preg_replace("/day/i", "d", $str);

	// normalize the input string. Replace all hour, hours with 'h'
	$str  = preg_replace("/hours/i", "h", $str);
	$str  = preg_replace("/hour/i", "h", $str);

	// now parse the string
	preg_match("/(\d+\.{0,1}\d*)\s*d/", $str, $matches);
	if ( count($matches) == 2 ) { 	// at least 2 entries are required
		$new_expire = (int)($matches[1] * 24);
		$valid = 1;
	}
	preg_match("/(\d+)\s*h/", $str, $matches);
	if ( count($matches) == 2 ) {	// at least 2 entries are required
		$new_expire += $matches[1];
		$valid = 1;
	}
	
	return $valid ? $new_expire : -1;

} // End of ParseExpire

/*
 * parse new max size.
 * valid form:
 *	<num>	defaults to num MB
 * 	<num><scale>	<scale> : K[B], M[B], G[B], and T[B]
 * returns the number of bytes of new max size or -1 if string unparsable
 */
function ParseMaxSize ($str) {

	$valid		 = 0;	// we found a valid string

	if ( $str == 'unlimited' ) 
		$str = "0";

	// a single number: add 'M'
	if ( is_numeric($str) ) {
		$str = (int)$str;
		$str .= "M";
	}

	// normalize the input string. Strip 'B'
	$str  = preg_replace("/B/", "", $str);

	preg_match("/(\d+\.{0,1}\d*)\s*([K|M|G|T])/", $str, $matches);
	if ( count($matches) == 3 ) {	// at least 3 entries are required
		if ( is_numeric($matches[1]) ) {
			$valid = 1;
		}
	}

	$str = preg_replace("/\s+/", '', $str);
	
	return $valid ? $str : '';

} // End of ParseMaxSize

?>
